
* 05_CustomFourLevelWorkBook.sas *;

******************************************************************;
* Module: CustomFourLevelWorkBook.sas                            *;
* Type: SAS sample program                                       *;
* Function: Open an Excel Workbook                               *;
* Load and Customize a four-level pre-formatted workbook with    *;
* an Overview worksheet (highest level totals + grand totals),   *;
* a Summary worksheet (all levels of subtotals + grand totals),  *;
* a Detail Plus Summary worksheet, and                           *;
* a Detail Only worksheet.                                       *;
* NOTE: It uses some macros available from the Author.           *;
* Author: LeRoy Bessler (bessler@execpc.com)                     *;
* Date of First Edition: 15 June 2003                            *;
******************************************************************;

* NOTE:
  To study the operation of this sample program,
  highlight specific code blocks and submit them individually.
  Once Excel has been started and a spreadsheet has been opened,
  you can toggle over to the Excel window after each SAS step
  to inspect the results.
  HOWEVER, the program can simply be submitted as a unit,
  and the spreadsheet processing will run start to finish.
  When done,
  you can inspect the SAS log, and go to inspect the workbook.
  Since the MPRINT option is turned on below,
  the SAS log will show you the code
  that is dynamically generated by the macros. *;

* These folders can, of course, be named anything you prefer,
  but the expected macros, input data, or output data must be stored there. *;
%let FolderForMacros    = C:\BesslerSASandExcel\macros;
%let FolderForXLSfiles  = C:\BesslerSASandExcel\spreadsheets;
%let FolderForXLSoutput = C:\BesslerSASandExcel\output;

* tell SAS where the macros used here are stored *;
filename SASandXL "&FolderForMacros";
* search for macros in the order listed *;
options sasautos=(SASandXL sasautos);

options mprint; * list the resolved translation of any macros
                  in the SAS log *;

* Prepare to create a worksheet that
  lists sashelp.class members
  by Name within Age category within Sex category,
  and, for (unrealistic) illustration purposes only,
  provides various Height and Weight subtotals and their grand totals. *;
* When there are three variables on the CLASS statement,
  PROC SUMMARY will provide eight (3 to the power 2) levels of totals:
  grand total,
  totals for each distinct value of each class variable,
  totals for each distinct pair of two class variable values,
  totals for each distinct triad of three class variable values. *;
* The various totals are identifiable by the _TYPE_ variable. *;
* You can get the full range of totals
  by removing the WHERE clause below. *;

proc summary data=sashelp.class;
class sex age name;
var height weight;
output sum=
       out=work.summed(drop=_freq_
           where=(_type_ in (7 6 4 0)));
run;

* To see what the summed dataset, with _type_ selection as above,
  yields, uncomment this PROC PRINT *;
* options nocenter;
* proc print data=work.summed;
* run;

* "Label" the various totals for identification in the worksheets,
  and add SortKeys to enable control of the order of output.
  Then do the sort. *;

data work.tosort(drop=_type_ sex age);
length sexNewVar $ 11;
length ageNewVar $ 8;
set work.summed;
if _type_ eq 0 then do;
  SortKey1 = 2;
  sexNewVar = 'Grand Total';
  output;
  return;
end;
SortKey1 = 1;
if _type_ eq 4 then do;
  SortKey3 = 2;
  sexNewVar = sex || ' Total';
  output;
  return;
end;
sexNewVar = sex;
SortKey3 = 1;
if _type_ eq 6 then do;
  SortKey5 = 2;
  ageNewVar = trim(left(age)) || ' Total';
  output;
  return;
end;
ageNewVar = left(age);
SortKey5 = 1;
output;
run;

proc sort data=work.ToSort
          out=work.ToReport(
              drop=SortKey1 SortKey3 SortKey5
              rename=(sexNewVar=Sex ageNewVar=Age));
by SortKey1 sexNewVar SortKey3 ageNewVar SortKey5 name;
run;

* To see what the ToReport dataset contains,
  uncomment this PROC PRINT *;
* options nocenter;
* proc print data=work.ToReport noobs;
* var sex age name height weight;
* run;

* Prepare datasets to feed each of the worksheets,
  and put macro variables into the global symbol table
  that will be used to customize the worksheets. *;

data work.DetailOnly;
set work.ToReport
    (where=(not (Age contains 'Total')
            and
            not (Sex contains 'Total')))
    end=lastone;
output;
if lastone;
call symput('LastRowForDetailOnly',trim(left(_N_ + 1)));
run;

proc sort data=work.DetailOnly;
by Name Sex Age;
run;

data work.DetailPlusSummary;
set work.ToReport end=lastone;
output;
if lastone;
call symput('LastRowForDetailPlusSummary',trim(left(_N_ + 1)));
run;

data work.SummaryColumns(keep=Sex Age Height Weight);;
set work.ToReport
    (where=(Age contains 'Total' or Sex contains 'Total'))
    end=lastone;
output;
if lastone;
call symput('LastRowForSummary',trim(left(_N_ + 1)));
run;

data work.OverviewColumns(keep=Sex Height Weight);
set work.ToReport
    (where=(Sex contains 'Total'))
    end=lastone;
output;
if lastone;
call symput('LastRowForOverview',trim(left(_N_ + 1)));
run;

* Start Excel *;
%XLStart;

* Define the fileref to be used to talk to Excel *;
%XLDDEcmdsFileName;

* Open the pre-formatted workbook *;
* The final result will be SAVED AS with a different XLS filename *;
%XLOpen(WorkBook=&FolderForXLSfiles.\EmptyViewsOfsasHelp_Class.xls);

* Load each of the four worksheets *;

* The column assignments for the variables
  are predicated on knowledge of how the empty worksheets
  have been "primed". *;
* You can go to look at them to see for yourself. *;
* Also, you can use Excel Point-and-Click
  to examine the pre-formatting choices that were made. *;

%XLFileNameForMultipleColumns(
           WorkSheet=Overview,
           StartRow=2,EndRow=&LastRowForOverview,
           StartColumn=1,EndColumn=3);

data _null_;
set work.OverviewColumns;
file Col1; put Sex;
file Col2; put Height;
file Col3; put Weight;
run;

%XLFileNameForMultipleColumns(
           WorkSheet=Summary,
           StartRow=2,EndRow=&LastRowForSummary,
           StartColumn=1,EndColumn=4);

data _null_;
set work.SummaryColumns;
file Col1; put Sex;
file Col2; put Age;
file Col3; put Height;
file Col4; put Weight;
run;

%XLFileNameForMultipleColumns(
           WorkSheet=Detail + Summary,
           StartRow=2,EndRow=&LastRowForDetailPlusSummary,
           StartColumn=1,EndColumn=5);

data _null_;
set work.DetailPlusSummary;
file Col1; put Sex;
file Col2; put Age;
file Col3; put Name;
file Col4; put Height;
file Col5; put Weight;
run;

%XLFileNameForMultipleColumns(
           WorkSheet=Detail Only,
           StartRow=2,EndRow=&LastRowForDetailOnly,
           StartColumn=1,EndColumn=5);

data _null_;
set work.DetailOnly;
file Col1; put Name;
file Col2; put Sex;
file Col3; put Age;
file Col4; put Height;
file Col5; put Weight;
run;

* Now customize the worksheets.
  Adjust the column width to the specific data values.
  Highlight the grand totals,
  whose row position could not be fore-ordained.
  Move the mouse to the upperleftmost data cell.
  Work on the Overview worksheet last,
  so that this is where the workbook opens. *;

* NOTE: With a little bit of extra pre-processing,
  and more use of the XLfont macro,
  it would also be possible to highlight the subtotals.
  Though the Sex subtotals could be sensibly highlighted,
  with such a small number of rows,
  to also highlight the Age subtotals would overload
  the Detail + Summary worksheet with too much Bold print. *;

%XLColumnWidth(WorkSheet=Detail Only,
               Column=1,ToColumn=5,Action=AutoFit);

* When this WorkSheet tab will be clicked on
  by someone who opens the WorkBook,
  assure that the mouse will be on the first data cell. *;

%XLFocus(Row=2,Column=1);

%XLSelect(WorkSheet=Detail + Summary,
          Row=&LastRowForDetailPlusSummary,
          ToRow=&LastRowForDetailPlusSummary,
          Column=1,ToColumn=5);

%XLFont(FontStyle=Bold);

* The appropriate worksheet has already been activated
  by the XLSelect macro *;

%XLColumnWidth(Column=1,ToColumn=5,Action=AutoFit);

%XLFocus(Row=2,Column=1);

* Highlight last summary row with Bold *;

%XLSelect(WorkSheet=Summary,
          Row=&LastRowForSummary,
          ToRow=&LastRowForSummary,
          Column=1,ToColumn=4);

%XLFont(FontStyle=Bold);

%XLColumnWidth(Column=1,ToColumn=4,Action=AutoFit);

%XLFocus(Row=2,Column=1);

%XLSelect(WorkSheet=Overview,
          Row=&LastRowForOverview,
          ToRow=&LastRowForOverview,
          Column=1,ToColumn=3);

%XLFont(FontStyle=Bold);

%XLColumnWidth(Column=1,ToColumn=3,Action=AutoFit);

%XLFocus(Row=2,Column=1);

* DO NOT overwrite the pre-formatted empty original. *;
%XLSaveAs(WorkBook=&FolderForXLSoutput.\MultipleViewsOfsasHelp_Class.xls);

* Finito *;
%XLExit;